@@ -1393,14 +1393,17 @@ class GenericMeta(TypingMeta, abc.ABCMeta):
 
     def __subclasscheck__(self, cls):
         if self.__origin__ is not None:
-            # This should only be modules within the standard
-            # library. singledispatch is the only exception, because
-            # it's a Python 2 backport of functools.singledispatch.
-            if sys._getframe(1).f_globals['__name__'] not in ['abc', 'functools',
-                                                              'singledispatch']:
-                raise TypeError("Parameterized generics cannot be used with class "
-                                "or instance checks")
-            return False
+            # These should only be modules within the standard library.
+            # singledispatch is an exception, because it's a Python 2 backport
+            # of functools.singledispatch.
+            whitelist = ['abc', 'functools', 'singledispatch']
+            if (sys._getframe(1).f_globals['__name__'] in whitelist or
+                    # The second frame is needed for the case where we came
+                    # from _ProtocolMeta.__subclasscheck__.
+                    sys._getframe(2).f_globals['__name__'] in whitelist):
+                return False
+            raise TypeError("Parameterized generics cannot be used with class "
+                            "or instance checks")
         if self is Generic:
             raise TypeError("Class %r cannot be used with class "
                             "or instance checks" % self)
@@ -1780,7 +1783,8 @@ class _ProtocolMeta(GenericMeta):
         if cls._is_protocol:
             for base in cls.__mro__[1:]:
                 if not (base in (object, Generic) or
-                        base.__module__ == '_abcoll' and base.__name__ in _PROTO_WHITELIST or
+                        base.__module__ == '_abcoll' and
+                        base.__name__ in _PROTO_WHITELIST or
                         isinstance(base, TypingMeta) and base._is_protocol or
                         isinstance(base, GenericMeta) and base.__origin__ is Generic):
                     raise TypeError('Protocols can only inherit from other protocols,'
@@ -1834,7 +1838,9 @@ class _ProtocolMeta(GenericMeta):
     def __subclasscheck__(self, cls):
         if (self.__dict__.get('_is_protocol', None) and
                 not self.__dict__.get('_is_runtime_protocol', None)):
-            if sys._getframe(1).f_globals['__name__'] in ['abc', 'functools']:
+            if (sys._getframe(1).f_globals['__name__'] in ['abc', 'functools'] or
+                    # This is needed because we remove subclasses from unions on Python 2.
+                    sys._getframe(2).f_globals['__name__'] == 'typing'):
                 return False
             raise TypeError("Instance and class checks can only be used with"
                             " @runtime_checkable protocols")
